import yaml
import traceback 
from libs import logs
import simplejson as json

log = logs.get_logger(__name__)

class Config:
    def __init__(self, **entries): 
        self.__dict__.update(entries)
    
    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__, ignore_nan=True)

def read_config(path, default = {}):
    try: 
        config = yaml.safe_load(open(path, 'r'))
        log.info('Read config %s: %s'%(path, config))
        return Config(**config);
    
    except Exception as error:
        log.error("No config file found. Return default config.")
        log.error("%s - %s" % (error, traceback.format_exc()))  
        return Config(**default);
        
def save_config(config, path, default = {}):
    try: 
        # config = yaml.dump(open(path, 'r'))
        with open(path, 'w') as file:
            yaml.dump(config, file, default_flow_style=False)
        log.info('Save config %s: %s'%(path, config))
        return True
    
    except Exception as error:
        log.error("No config file found. Return default config.")
        log.error("%s - %s" % (error, traceback.format_exc()))  
        return Config(**default);


import os
import traceback

from libs.message_handler import BaseMessageHandler
from libs.message import ContentType

from libs import logs
from libs.message import ProjectCommand
from project_manager import files, projects

log = logs.get_logger(__name__)


class MessageHandler(BaseMessageHandler):
    def __init__(self, p2n_queue, user_space=None):
        super(MessageHandler, self).__init__(p2n_queue, user_space)

    def handle_message(self, message):
        log.info('Handle FileExplorer message: %s' % message)
        try:
            metadata = message.metadata
            if 'path' in metadata.keys():
                ## avoid creating `./` when the path is empty
                if metadata['path']=="":
                    norm_path = metadata['path']
                else:
                    norm_path = os.path.normpath(metadata['path'])
            if 'project_path' in metadata.keys():
                norm_project_path = os.path.normpath(metadata['project_path'])

            output = None
            if message.command_name == ProjectCommand.list_dir:
                output = []
                if 'path' in metadata.keys() and 'project_path' in metadata.keys():
                    output = files.list_dir(norm_project_path, norm_path)
                    type = ContentType.DIR_LIST
            elif message.command_name == ProjectCommand.create_file:
                if 'path' in metadata.keys() and 'project_path' in metadata.keys():
                    files.create_file(norm_project_path, norm_path)
                output = projects.open_file(metadata['path'])
                type = ContentType.FILE_METADATA
            elif message.command_name == ProjectCommand.delete:
                if 'path' in metadata.keys() and 'project_path' in metadata.keys():
                    files.delete(norm_project_path, norm_path)

                    # Delete state file
                    state_file_path = files.get_state_path(
                        norm_project_path, norm_path)
                    if os.path.exists(state_file_path):
                        files.delete(norm_project_path, state_file_path)
                if ('is_file' in metadata) and metadata['is_file']:
                    output = projects.close_file(metadata['path'])
                else:  # TODO: handle the case where a dir is deleted and deleted files were opened
                    output = projects.get_open_files()
                type = ContentType.FILE_METADATA
            # create reply message
            message.type = type
            message.content = output
            message.error = False
            self._send_to_node(message)

        except:
            trace = traceback.format_exc()
            log.error("%s" % (trace))
            error_message = MessageHandler._create_error_message(
                message.webapp_endpoint, trace)
            self._send_to_node(error_message)



# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

import ipywidgets as widgets
from ipywidgets.widgets.widget_link import Link

from traitlets import CaselessStrEnum, Unicode, Tuple, List, Bool, CFloat, Float, CInt, Int, Instance, Dict, Any
from ipywidgets import Color
from ipywidgets.widgets.trait_types import TypedTuple

header = '''# Model State

This is a description of the model state for each widget in the core Jupyter widgets library. The model ID of a widget is the id of the comm object the widget is using. A reference to a widget is serialized to JSON as a string of the form `"IPY_MODEL_<MODEL_ID>"`, where `<MODEL_ID>` is the model ID of a previously created widget of the specified type.

This model specification is for ipywidgets 7.4.*, @jupyter-widgets/base 1.1.*, and @jupyter-widgets/controls 1.4.*.

## Model attributes

Each widget in the Jupyter core widgets is represented below. The heading represents the model name, module, and version, view name, module, and version that the widget is registered with.

'''

widgets_to_document = sorted(widgets.Widget.widget_types.items())

def typing(x):
    s = ''
    if isinstance(x, CaselessStrEnum):
        s = 'string (one of %s)'%(', '.join('`%r`'%i for i in x.values))
    elif isinstance(x, Unicode):
        s = 'string'
    elif isinstance(x, (Tuple, List)):
        s = 'array'
    elif isinstance(x, TypedTuple):
        s = 'array of ' + typing(x._trait)
    elif isinstance(x, Bool):
        s = 'boolean'
    elif isinstance(x, (CFloat, Float)):
        s = 'number (float)'
    elif isinstance(x, (CInt, Int)):
        s = 'number (integer)'
    elif isinstance(x, Color):
        s = 'string (valid color)'
    elif isinstance(x, Dict):
        s = 'object'
    elif isinstance(x, Instance) and issubclass(x.klass, widgets.Widget):
        s = 'reference to %s widget'%(x.klass.__name__)
        # ADD the widget to this documenting list
        if x.klass not in [i[1] for i in widgets_to_document] and x.klass != widgets.Widget:
            widgets_to_document.append((x.klass.__name__, x.klass))
    elif isinstance(x, Any):
        # In our case, these all happen to be values that are converted to strings
        s = 'string (valid option label)'
    else:
        s = x.__class__.__name__
    if x.allow_none:
        s = "`null` or "+s
    return s

def jsdefault(t):
    x = t.default_value
    if isinstance(t, Instance):
        x = t.make_dynamic_default()
        if issubclass(t.klass, widgets.Widget):
            return 'reference to new instance'
    if x is True:
        return '`true`'
    elif x is False:
        return '`false`'
    elif x is None:
        return '`null`'
    elif isinstance(x, tuple):
        return '`{0}`'.format(list(x))
    else:
        return '`%s`'%t.default_value_repr()

def format_widget(n, w):
    out = []
    name = dict(zip(['m_module', 'm_version', 'model', 'v_module', 'v_version', 'view'], n))

    out.append('### %(model)s (%(m_module)s, %(m_version)s); %(view)s (%(v_module)s, %(v_version)s)'%name)
    out.append('')
    out.append('{name: <16} | {typing: <16} | {default: <16} | {help}'.format(name='Attribute', typing='Type', 
                                                                              default='Default', help='Help'))
    out.append('{0:-<16}-|-{0:-<16}-|-{0:-<16}-|----'.format('-'))
    for name, t in sorted(w.traits(sync=True).items()):
        if name in ('_model_module', '_view_module', '_model_module_version', '_view_module_version', 
                    '_dom_classes', 'layout'):
            # document these separately, since they apply to all classes
            pass
        if name in ('_view_count'):
            # don't document this since it is totally experimental at this point
            continue

        s = '{name: <16} | {typing: <16} | {default: <16} | {help}'.format(name='`%s`'%name, typing=typing(t), 
                                                            allownone='*' if t.allow_none else '', 
                                                                                               default=jsdefault(t),
                                                                                              help=t.help if t.help else '')
        out.append(s)
    out.append('')
    return '\n'.join(out)

out = header
for n,w in widgets_to_document:
    if issubclass(w, Link):
        out += '\n'+format_widget(n, w((widgets.IntSlider(), 'value'), (widgets.IntSlider(), 'value')))
    elif issubclass(w, widgets.SelectionRangeSlider) or issubclass(w, widgets.SelectionSlider):
        out += '\n'+format_widget(n,w(options=[1]))
    else:
        out += '\n'+format_widget(n,w())
print(out)



import os
import sys
from subprocess import Popen
from contextlib import contextmanager
import time


web_path = os.path.dirname(os.path.realpath(__file__))  # cyc-next
server_path = os.path.join(web_path, 'server')
FILE_NAME = '.server.yaml'
CHANGE_LINE_NUMBER = 14
FIELD_LENGTH = 10


def change_path(path):
    os.chdir(server_path)
    my_file = open(FILE_NAME)

    string_list = my_file.readlines()
    # Get file's content as a list
    my_file.close()
    content = string_list[CHANGE_LINE_NUMBER]
    print(string_list[CHANGE_LINE_NUMBER])
    string_list[CHANGE_LINE_NUMBER] = content[:FIELD_LENGTH] + \
        'path: ' + '\'' + path + '\'' + '\n'
    print(string_list[CHANGE_LINE_NUMBER])
    my_file = open(FILE_NAME, 'w')
    new_file_contents = ''.join(string_list)
    # Convert `string_list` to a single string
    my_file.write(new_file_contents)
    my_file.close()
    print('map path done!')


def main():
    path()
    os.chdir(web_path)
    os.system('npm i --force')
    os.system('npm run build')
    os.chdir(server_path)
    os.system('npm i')


def path():
    path = input('Please enter your project directory\'s path:')
    print('Checking your path: ' + path)
    if os.path.isdir(path):
        os.chdir(path)
        folder_name = os.path.basename(path)
        print('folder_name', folder_name)
        change_path(path)
    else:
        print('Your path isn\'t correct, Please try again')


@contextmanager
def run_and_terminate_process():
    try:
        print("cnext starting !")

        os.chdir(web_path)
        web_proc = Popen('npm start', shell=True)

        os.chdir(server_path)
        my_env = os.environ.copy()
        my_env["PATH"] = os.path.dirname(
            sys.executable) + os.path.pathsep + my_env["PATH"]

        ser_proc = Popen('npm run start-prod', shell=True, env=my_env)
        yield

    finally:
        web_proc.terminate()  # send sigterm, or ...
        web_proc.kill()      # send sigkill

        ser_proc.terminate()  # send sigterm, or ...
        ser_proc.kill()      # send sigkill


def start():
    with run_and_terminate_process() as running_proc:
        while True:
            time.sleep(1000)


if __name__ == '__main__':
    main()



import base64
import traceback
from libs.message_handler import BaseMessageHandler
from libs.message import ContentType, DFManagerCommand, SubContentType
from cnextlib.mime_types import CnextMimeType
import simplejson as json

from libs import logs
from user_space.user_space import ExecutionMode
log = logs.get_logger(__name__)


class MessageHandler(BaseMessageHandler):
    def __init__(self, p2n_queue,  user_space=None):
        super(MessageHandler, self).__init__(p2n_queue, user_space)

    #TODO: unify this with _create_plot_data
    def _create_plot_data(self, df_id, col_name, result):
        return {'df_id': df_id, 'col_name': col_name, 'plot': result.to_json()}

    def _get_mime_file_content(self, file_path, mime_type):
        with open(file_path, 'rb') as file:
            return file.read()

    def _create_table_data(self, df_id, df):
        ''' The tableData will be the same as the original `df` except for two exceptions
            1. if column type is datatime64[ns], the data will be reformated
            2. if the column type is mime type such as file/png, we store the data of 
            that field in  dictionary of this format {file_path: file_path, binary: binary_data}.
            This is to ensure that we have access to both original data in the dataframe and the 
            binary data in the frontend for this special case'''
        tableData = {}
        tableData['df_id'] = df_id
        log.info('DF Columns', df.columns)
        tableData['column_names'] = list(df.columns)

        ## Convert datetime to string so it can be displayed in the frontend #
        for i, t in enumerate(df.dtypes):
            if t.name == 'datetime64[ns]':
                df[df.columns[i]] = df[df.columns[i]
                                       ].dt.strftime('%Y-%m-%d %H:%M:%S')

        tableData['rows'] = df.values.tolist()
        ## Modify data field of column with mime type of file/*. See note above
        # We chose to do this outside of the dataframe, but it can also be done with a dataframe
        # see https://stackoverflow.com/questions/41710501/is-there-a-way-to-have-a-dictionary-as-an-entry-of-a-pandas-dataframe-in-python #
        for i, t in enumerate(df.dtypes):
            if t.name in [CnextMimeType.FILE_PNG, CnextMimeType.FILE_JPG]:
                log.info('Load file for mime %s' % t.name)
                for r in range(df.shape[0]):
                    file_path = df[df.columns[i]].iloc[r]
                    log.info('Load file for mime %s path %s' %
                             (t.name, file_path))
                    tableData['rows'][r][i] = {
                        'file_path': file_path,
                        'binary': base64.b64encode(self._get_mime_file_content(file_path, t.name))
                    }

        tableData['index'] = {}
        tableData['index']['name'] = df.index.name
        tableData['index']['data'] = []
        if str(df.index.dtype) == 'datetime64[ns]':
            [tableData['index']['data'].append(str(idx)) for idx in df.index]
        else:
            tableData['index']['data'] = df.index.tolist()
        # log.info(tableData)
        return tableData

    def _create_countna_data(self, df_id, len, countna_series):
        countna = {}
        for k, v in countna_series.to_dict().items():
            countna[k] = {'na': v, 'len': len}
        return {'df_id': df_id, 'countna': countna}

    # def _get_file_content(self, message, client_globals):
    #     message_content: LoadFileMessageContent = LoadFileMessageContent(message.content)
    #     file_content = None
    #     with open(message_content.file_path, 'rb') as file:
    #         file_content = file.read()
    #     output = {'file_content': file_content, 'mime_type': message_content.mime_type}
    #     return output, ContentType.BINARY, True

    def _get_count_na(self, df_id):
        output = None
        countna = self.user_space.execute(
            "%s.isna().sum()" % df_id, ExecutionMode.EVAL)
        len = self.user_space.execute(
            "%s.shape[0]" % df_id, ExecutionMode.EVAL)
        if (countna is not None) and (len is not None):
            log.info("get countna data")
            output = self._create_countna_data(df_id, len, countna)
        return output

    def _get_table_data(self, df_id, code):
        output = None
        result = self.user_space.execute(code, ExecutionMode.EVAL)
        if result is not None:
            log.info("get table data %s" % result)
            output = self._create_table_data(df_id, result)
        return output

    def _get_metadata(self, df_id):
        shape = self.user_space.execute(
            "%s.shape" % df_id, ExecutionMode.EVAL)
        dtypes = self.user_space.execute(
            "%s.dtypes" % df_id, ExecutionMode.EVAL)
        countna = self.user_space.execute(
            "%s.isna().sum()" % df_id, ExecutionMode.EVAL)
        describe = self.user_space.execute(
            "%s.describe(include='all')" % df_id, ExecutionMode.EVAL)
        columns = {}
        for col_name, ctype in dtypes.items():
            # print(col_name, ctype)
            # FIXME: only get at most 100 values here, this is hacky, find a better way
            # unique = eval("%s['%s'].unique().tolist()"%(df_id, col_name), client_globals)[:100]
            unique = self.user_space.execute(
                "%s['%s'].unique().tolist()" % (df_id, col_name), ExecutionMode.EVAL)
            columns[col_name] = {'name': col_name, 'type': str(ctype.name), 'unique': unique,
                                 'describe': describe[col_name].to_dict(), 'countna': countna[col_name].item()}
        output = {'df_id': df_id, 'shape': shape, 'columns': columns}
        return output

    def handle_message(self, message):
        send_reply = False
        # message execution_mode will always be `eval` for this sender
        log.info('eval... %s' % message)
        # log.info('Globals: %s' % client_globals)
        try:
            if message.command_name == DFManagerCommand.plot_column_histogram:
                # result = eval(message.content, client_globals)
                result = self.user_space.executor.execute(
                    message.content, ExecutionMode.EVAL)
                if result is not None:
                    log.info("get plot data")
                    output = self._create_plot_data(
                        message.metadata['df_id'], message.metadata['col_name'], result)
                    # type = ContentType.PLOTLY_FIG
                    type = ContentType.RICH_OUTPUT
                    sub_type = SubContentType.IMAGE_PLOTLY
                    send_reply = True

            elif message.command_name == DFManagerCommand.plot_column_quantile:
                # result = eval(message.content, client_globals)
                result = self.user_space.executor.execute(
                    message.content, ExecutionMode.EVAL)
                if result is not None:
                    log.info("get plot data")
                    output = self._create_plot_data(
                        message.metadata['df_id'], message.metadata['col_name'], result)
                    # type = ContentType.PLOTLY_FIG
                    type = ContentType.RICH_OUTPUT
                    sub_type = SubContentType.IMAGE_PLOTLY
                    send_reply = True

            elif message.command_name == DFManagerCommand.get_table_data:
                output = self._get_table_data(
                    message.metadata['df_id'], message.content)
                if output is not None:
                    type = ContentType.PANDAS_DATAFRAME
                    sub_type = SubContentType.NONE
                    send_reply = True

            elif message.command_name == DFManagerCommand.get_countna:
                output = self._get_count_na(message.metadata['df_id'])
                if output is not None:
                    log.info("get countna data")
                    type = ContentType.DICT
                    sub_type = SubContentType.NONE
                    send_reply = True

            elif message.command_name == DFManagerCommand.get_df_metadata:
                output = self._get_metadata(message.metadata['df_id'])
                type = ContentType.DICT
                sub_type = SubContentType.NONE
                send_reply = True

            # elif message.command_name == DFManagerCommand.get_file_content:
            #     output, type, send_reply = self._get_file_content(message, client_globals)

            if send_reply:
                message.type = type
                message.sub_type = sub_type
                message.content = output
                message.error = False
                self._send_to_node(message)
        except:
            trace = traceback.format_exc()
            log.error("%s" % (trace))
            error_message = MessageHandler._create_error_message(
                message.webapp_endpoint, trace)
            self._send_to_node(error_message)



import logging

logging.basicConfig(filename='./server.log', filemode='a', format='%(asctime)s,%(msecs)d %(name)s %(funcName)s %(levelname)s %(message)s', 
                        datefmt='%H:%M:%S', level=logging.DEBUG)

def get_logger(name):
    return logging.getLogger(name)